/* affap.h -- Affine approximation to functions in given interval. */

#include <flt.h>
#include <ia.h>

/* The procedures below compute affine approximations to various 
  unary functions "f(x)" over a specified argument range.
  
  In each case, the argument range is specified by its center value
  "xc" and the half-width "xr".  The resulting affine approximation
  $g(x)" is "alpha*(x-xc)/zeta + gamma", where "alpha", "zeta", and "gamma"
  are returned by the procedure.  The procedure also returns an upper
  bound "delta" for the absolute error "abs(g(x)-f(x))" for any "x" in "xr".
  
  The returned values of "alpha", "zeta", and "gamma" are always finite.
  If overflow occurs during their computation, the precedure returns
  "alpha = zeta = One", "gamma = 0", and "delta = PlusInfinity".
*/

void affap_sqrt
  ( Float xc,
    Float xr,
    Floatp alphap,
    Floatp zetap,
    Floatp gammap,
    Floatp deltap
  );

