/* Last edited on 2010-05-24 01:45:33 by stolfi */ 

/* ---------------------------------------------------------------------- */
/* mst.h */

int mst_find_root(int R[], int u);
  /* The "find" operation of Union-Find: Finds the root of the tree
    that contains the vertex {u}, by following the shortcut map {R}
    until reaching a vertex {v} such that {R[v]=v}. In the process,
    changes {R[w]} of every visited vertex {w} to point directy to
    {u}, thus making {u} the root of the tree. */
    
/* ---------------------------------------------------------------------- */
/* spmat.c */

spmat_t *spmat_cast_ref(void *M);
  /* Casts the argument as an address to a {spmat_t}. Can be used to
    apply spmat_expand or spmat_trim to sparse matrices of specific types. 
    We define it as a function in order to trigger type warnings 
    when {M} is not an address (a common mistake). */


/* ---------------------------------------------------------------------- */
/* ulist.c */
/* Picking a random non-NULL item in a sequential hash table. */

ulist_item_t ulist_pick(ulist_t *S)
  { ulist_count_t sz = S->it.nel;
    if (sz == 0) { return NULL; }
    ulist_index_t k = abrandom(0, sz-1);
    if (S->it.el[k] != NULL) { return S->it.el[k]; }
    if (sz == 1) { return NULL; }
    /* Choose a good stride {d} rel prime to {sz}: */
    ulist_count_t d = (int)(0.618034 * (double)sz);
    while (gcd(d,sz) != 1) { d--; }
    ulist_count_t i = k + d % sz;
    do {
      ulist_item_t a = S->it.el[i];
      if (a != NULL) { return a; }
      i = (i + d) % sz;
    } while (i != k);
    return NULL;
    
    /* 
      gawk \
        ' BEGIN { \
            for (n = 2; n < 1000; n++) \
              { d0 = int(0.618034 * n); \
                d = d0; \
                while (gcd(d,n) > 1) { d--; } \
                printf "%5d %5d %5d %5d\n", n, d0, d, d0 - d; \
              } \
          } \
          function gcd(a,b,  r) \
          { while (b != 0) \
              { r = a % b; a = b; b = r; } \
            return a; \
          } \
        '
    */
  }
  
/* ---------------------------------------------------------------------- */
/* ulist.c */

void ulist_clear_all_slots(ulist_item_vec_t *it);
  /* Sets all slots of {it} to NULL. */

void ulist_clear_all_slots(ulist_item_vec_t *it)
  { unsigned int i;
    for (i = 0; i < it->nel; i++) { it->el[i] = NULL; }
  }

/* ---------------------------------------------------------------------- */
/* indexing.h, indexing.c */
bool_t ix_next_C ( ix_dim_t d, ix_index_t ix[], ix_size_t sz[], ix_step_t st[], ix_pos_t *p );
  /* Set {ix[0..d-1]} to the next valid index tuple in the C/Pascal
    standard order (with the last index in the innermost loop).
    
    More precisely, tries to increment {ix[d-1]} first. If an index
    {ix[i]} to be incremented has reached its limit {sz[i]-1}, resets
    all indices {ix[i..d-1]} to 0, and tries to increment index
    {ix[i-1]} instead. If this `carry over' falls off the index tuple
    -- that is, if the tuple {ix} changes from {sz - (1,..1)} back to
    {(0,..0)} -- the procedure returns TRUE; otherwise it returns
    FALSE. The effect is undefined if any {sz[i]} is 0 or {ix[i]}
    is not in {0..sz[i]-1}.
    
    If {p} is not null, assumes that {*p} is the position corresponding
    to the index tuple {ix}, and adjusts it to account for the change.
  
    This procedure can be used to vary the index tuple {ix[0..d-1]}
    over all the elements of an array, in a single loop, e.g.
    
      | if (ix_assign_min(d,ix,sz))
      |   { ix_pos_t p = ix_position(d,ix,bp,st);
      |     do { ... } while (! ix_next_C(d,ix,sz,st,&p));
      |   }

    */

bool_t ix_next_C ( ix_dim_t d, ix_index_t ix[], ix_size_t sz[], ix_step_t st[], ix_pos_t *p )
  { /* Look for an index {ix[i]} that can be incremented: */
    int i = d-1;
    ix_index_t *ixi = ix + i;
    while ((*ixi) >= sz[i]-1) 
      { /* Index {ix[i]} is at its limit, reset to 0: */
        if (p != NULL) { (*p) -= (*ixi)*st[i]; }
        (*ixi) = 0;
        /* Wrap around? */
        if (i <= 0) { return TRUE; }
        /* Go to next index: */
        i--; ixi--;
      }
    /* Increment index {ix[i]} */
    (*ixi)++; 
    if (p != NULL) { (*p) += st[i]; }
    return FALSE;
  }
    
bool_t ix_prev_C ( ix_dim_t d, ix_index_t ix[], ix_size_t sz[], ix_step_t st[], ix_pos_t *p )
  { /* Look for an index {ix[i]} that can be decremented: */
    int i = d-1;
    ix_index_t *ixi = ix + i;
    while ((*ixi) <= 0) 
      { /* Index {ix[i]} is at its limit, reset to {sz[i]-1}: */
        (*ixi) = sz[i]-1;
        if (p != NULL) { (*p) += (*ixi)*st[i]; }
        /* Wrap around? */
        if (i <= 0) { return TRUE; }
        /* Go to next index: */
        i--; ixi--;
      }
    /* Decrement index {ix[i]} */
    (*ixi)--;
    if (p != NULL) { (*p) -= st[i]; }
    return FALSE;
  }
  
sign_t ix_compare_C ( ix_dim_t d, ix_index_t ixa[], ix_index_t ixb[] )
  { int i;
    ix_index_t *ixai = ixa, *ixbi = ixb;
    for (i = 0; i < d; i++)
      { if ((*ixai) < (*ixbi))
          { return NEG; }
        else
          { return POS; }
        ixai++; ixbi++;
      }
    return ZER;
  }



/* ---------------------------------------------------------------------- */
/* jstime.c */

#if (defined(SunOS4))
#warning "implementing {now} with {getrusage(...)}."
double now(void) 
  { struct rusage ru;
    getrusage(RUSAGE_SELF, &ru);
    return(((double)ru.ru_utime.tv_sec)*1000000.0 + ((double)ru.ru_utime.tv_usec));
  }
#endif

#if (defined(SunOS5))
#warning "implementing {now} with {times(...)}."
extern long int __sysconf (int);
double now(void) 
  { struct tms buf;
    clock_t t;
    times(&buf);
    t = buf.tms_utime;
    return(1000000.0L * ((double) t)/((double) _sysconf(3)));
  }
#endif
 
#if (defined(OSF1V4))
#warning "implementing {now} with {times(...)}."
double now(void) 
  { struct tms buf;
    clock_t t;
    times(&buf);
    t = buf.tms_utime;
    return(1000000.0L * ((double) t)/((double) CLK_TCK));
  }
#endif

/* ---------------------------------------------------------------------- */
                /* Locate {jMin,jMax} near {iTry} s.t. */
                /* {iMin <= jMin <= jMax <= iMax} and {f(jMin) < 0 <= f(jMax)}: */
                int jMin = iTry-1; int jMax = iTry; int d = 1;
                if (TB_DEBUG) { fprintf(stderr, "!<%d:%d>", jMin,jMax); }
                double gMin = f(jMin);
                while ((jMin > iMin) && (gMin >= 0)) 
                  { jMax = jMin;
                    jMin -= d; if (jMin < iMin) { jMin = iMin; }
                    gMin = (jMin == iMin ? fMin : f(jMin));
                    d += d;
                    if (TB_DEBUG) fprintf(stderr, "-");
                    if (TB_DEBUG) fprintf(stderr, "!<%d:%d>", jMin,jMax);
                  }
                double gMax = f(jMax);
                while ((jMax < iMax) && (gMax < 0)) 
                  { jMin = jMax; 
                    jMax += d; if (jMax > iMax) { jMax = iMax; }
                    gMax = (jMax == iMax ? fMax : f(jMax));
                    d += d;
                    if (TB_DEBUG) fprintf(stderr, "+");
                    if (TB_DEBUG) fprintf(stderr, "!<%d:%d>", jMin,jMax);
                  }
                if (jMin != iMin) { iMin = jMin;  fMin = f(iMin); }
                if (jMax != iMax) { iMax = jMax;  fMax = f(iMax); }
                demand (fMin <= fMax, "tbl out of order");


/* ---------------------------------------------------------------------- */
void srt_bins_sort(int *h, int n, int cmp(int x, int y), int sgn)
  { int m;
    for (m = 1; m < n; m++)
      { /* Insert {h[m]} among {h[0..m-1]}: */
        int v = h[m], i = m;
        while ((i > 0) && (sgn*cmp(v, h[i-1]) < 0)) 
          { h[i] = h[i-1]; i--; }
        h[i] = v;
      }
  }
/* ---------------------------------------------------------------------- */
void ix_indices ( ix_dim_t d, ix_disp_t pos, ix_disp_t b, ix_disp_t s[], ix_index_t e[] );
  /* Computes the indices {e[0,..d-1]} of an element given its 
    position {pos}. Assumes that the steps {s[0..i-1]} are graded
    (see {ix_step_valid} below). May fail if {pos} is not a valid
    position computable from {b} and {s}. */

/* ---------------------------------------------------------------------- */
/* INTERNAL PROTOS */

void ix_progerror 
  ( const char *msg, 
    const char *file, 
    const unsigned int line, 
    const char* proc 
  );
  /* Prints {file ":" line ": (" *proc ")" *msg} to {stderr} and stops. */

#define ix_error(msg) \
  ix_progerror((msg), __FILE__, __LINE__, __FUNCTION__)
  /* Prints {*msg} and the source location.  */

/* ---------------------------------------------------------------------- */

/* ERROR MESSAGES */
  
void ix_progerror 
  ( const char *msg, 
    const char *file, 
    const unsigned int line, 
    const char* proc 
  )
  { fprintf (stderr, "%s:%u: *** (%s) %s\n", file, line, proc, msg);
    exit(1);
  }
/* ---------------------------------------------------------------------- */
/* INDEX ROLL OPERATION - CORRECT BUT TOO CONFUSING TO BE USEFUL */

void ix_roll
  ( ix_dim_t d, 
    ix_size_t sz[], 
    ix_pos_t *bp, 
    ix_step_t st[], 
    ix_axis_t i,
    ix_axis_t j,
    int n
  );
  /* Rotates indices {i..j} by {n} places forward.
  
    The procedure moves {sz[k]} to {sz[k']} and {st[k]} to {st[k']} for
    all {k} in {i..j} simultaneously, where {k'} is {k + n} reduced
    cyclically to the range {i..j} --- that is, {k'=i+(k-i+n)mod(j-i+1)}. 
    Thus, {ix[k] = ix'[k']} for all {k} in {i..j}, and {ix[k]=ix'[k]} for
    all other {k}.
    
    The displacement {n} may be negative. For example, to eliminate a
    trivial index {i} from a matrix, use {ix_roll(d,sz,&bp,st,i,d-1,-1);
    d--;}. The procedure has no effect if {n==0} or {i==j}. Undefined
    if {i > j}. */

void ix_roll
  ( ix_dim_t d, 
    ix_size_t sz[], 
    ix_pos_t *bp, 
    ix_step_t st[], 
    ix_axis_t i,
    ix_axis_t j,
    int n
  )
  { /* Check axes: */
    demand(i < d, "bad axis i");
    demand(j < d, "bad axis j");
    demand(i <= j, "bad axis interval");
    /* Check for trivial roll: */
    if ((i == j) || (n == 0)) { return; }
    /* Number of elements to be rolled: */
    int r = j - i + 1;
    /* Reduce {n} modulo {r}: */
    n = n % r; if (n < 0) { n += r; }
    /* Remaining candidate cycle heads are {i+kini..i+klim-1}: */
    int kini = 0, klim = r;
    while (kini < klim)
      { /* Save cycle head {i+kini}: */
        ix_size_t tm = sz[i + kini];
        ix_pos_t  ts = st[i + kini];
        /* Shift elems in cycle starting at {i+kini}: */
        int ka = kini;
        int kb;
        do 
          { kb = (ka + r - n) % r;
            sz[i + ka] = sz[i + kb]; 
            st[i + ka] = st[i + kb];
            ka = kb; 
            /* Update {klim} (should happen in first cycle only): */
            if (kb < klim) { klim = kb; }
          }
        while (ka != kini); 
        /* Store cycle head: */
        sz[i + kb] = tm;
        st[i + kb] = ts;
        /* Get next cycle head: */
        kini++;
      }
    /* Exchange parameters of axes {i,j}: */
  }

/* ---------------------------------------------------------------------- */
