/* Last edited on 2009-08-25 13:25:15 by stolfi */
  
void pswr_color_scale_3
  ( double fs, double ft, double fu, 
    double Ymin,
    double Ymax,
    double *R, double *G, double *B
  )
  {
    /* Choose three equally spaced zero-brightness vectors: */
    double sR = +0.63089, sG = -0.36911, sB = +0.25143;
    double tR = -0.44074, tG = +0.05926, tB = +0.84696;
    double uR = -0.19015, uG = +0.30985, uB = -1.09838;
    /* Compute the brightness {Y} of the target color: */ 
    double fw = (fs+ft+fu)/M_SQRT3;  /* Diagonal coord of {fs,ft,fu}. */
    double fZ = (fw + 1)/2; /* Pseudo-brightness of {fs,ft,fu}. */
    double bias = 0.125;
    double r = (Ymax + bias)/(Ymin + bias);
    double Y = (Ymin + bias)*exp(fZ*log(r)) - bias;
    if (Y < 0) { Y = 0; }
    if (Y > 1) { Y = 1; }
    /* Compute the pseudo-saturation {m} of {fs} relative to the unit 3-ball: */ 
    double m2 = fs*fs + ft*ft + fu*fu - fw*fw;
    if (m2 < 0) { m2 = 0; }
    double m = sqrt(m2);
    /* Eliminate the {fw} component from {fs,ft,fu}: */
    fs -= fw/M_SQRT3; ft -= fw/M_SQRT3; fu -= fw/M_SQRT3;
    /* Compute the limiting chroma direction vector (for {m = 1}): */
    double Rd = fs/m * sR + ft/m * tR + fu/m * uR;
    double Gd = fs/m * sG + ft/m * tG + fu/m * uG;
    double Bd = fs/m * sB + ft/m * tB + fu/m * uB;
    /* Compute the maximum {h <= 1} s.t. {(Y,Y,Y) + h*(Rd,Gd,Bd)} is ok: */
    double h = 1.0, lim;
    lim = (Rd < 0 ? 0 : 1) - Y; if (h*Rd*Rd > lim*Rd) { h = lim/Rd; }
    lim = (Gd < 0 ? 0 : 1) - Y; if (h*Gd*Gd > lim*Gd) { h = lim/Gd; }
    lim = (Bd < 0 ? 0 : 1) - Y; if (h*Bd*Bd > lim*Bd) { h = lim/Bd; }
    /* Now add the chroma component to {(Y,Y,Y)}: */
    m *= h; 
    (*R) = Y + m*Rd;
    (*G) = Y + m*Gd;
    (*B) = Y + m*Bd;
  }
